pub enum SetMutation {
Insert(usize),
Remove(usize),
}Expand description
Set mutation operations.
See the InclusionExclusion struct.
Variants
Insert(usize)
Add the item with the given index to the set.
Remove(usize)
Remove the item with the given index from the set.