pub struct InclusionExclusion { /* private fields */ }Expand description
Iterator yielding SetMutations that can be used to efficiently
visit all subsets of n items
The pattern of use is to in-place mutate a working set by
following the SetMutation instructions, starting from an
initially empty set. Every subset will be generated, none will be
repeated, and only a single mutation per iteration is necessary.
Examples
Visit every subset of a set of strings by mutating a HashSet.
use std::collections::HashSet;
use gray_codes::{InclusionExclusion, SetMutation};
static STRINGS: &[&str] = &["apple", "moon", "pie"];
let mut subset = HashSet::new();
// Visit the empty set here, if desired.
println!("{:?}", subset);
for mutation in InclusionExclusion::of_len(STRINGS.len()) {
// Mutate the set, according to instructions.
match mutation {
SetMutation::Insert(i) => { subset.insert(STRINGS[i]); },
SetMutation::Remove(i) => { subset.remove(STRINGS[i]); },
}
// Visit the unique subset here.
println!("{:?}", subset);
}Iterate over the 15 possible ways to sum four numbers using only a single addition or subtraction per iteration. This technique can be used to brute-force the subset sum problem faster than the naive, obvious way.
use gray_codes::{InclusionExclusion, SetMutation};
let addends = [235, 63, 856, 967];
let mut sum = 0;
for mutation in InclusionExclusion::of_len(addends.len()) {
match mutation {
SetMutation::Insert(i) => { sum += addends[i]; },
SetMutation::Remove(i) => { sum -= addends[i]; }
}
// Process the sum somehow.
println!("{}", sum);
}Implementations
Sourceimpl InclusionExclusion
impl InclusionExclusion
Sourcepub fn of_len(item_count: usize) -> InclusionExclusion ⓘ
pub fn of_len(item_count: usize) -> InclusionExclusion ⓘ
Constructor for an InclusionExclusion iterator over the
given number of objects.
See the struct documentation for examples.
Panics
This method panics if either of the following conditions are true:
item_countis zero.item_countuses more than 16 bits.