Struct gray_codes::Subsets [] [src]

pub struct Subsets<'a, T: 'a, C> { /* fields omitted */ }

Iterator yielding subsets of a slice

Use the static method Subsets::of(...) to construct the iterator.

The input is a slice of type &'a [T] and the output is any container C that is FromIterator<&'a T>. In many cases, it's good enough for the collection C to be Vec<&'a T>, in which case you can use the convenient VecSubsets type alias.

A new C-container is created every iteration, which is an O(set_len) operation per iteration. Greater efficiency (O(1) per iteration) can be gained by using the InclusionExclusion iterator to perform mutation directly on your own container.

Examples

Collect every subset of 0..4 into a Vec of Vecs.

use gray_codes::VecSubsets;
static NUMBERS: &[u32] = &[0,1,2,3];
let mut subsets: Vec<_> = VecSubsets::of(NUMBERS).collect();
assert!(subsets.len() == 16);
subsets.sort();
// (Note that this is sorted order, not the order in which the
// Subsets iterator generates items.)
assert!(subsets == vec![vec![],
                        vec![&0],
                        vec![&0,&1],
                        vec![&0,&1,&2],
                        vec![&0,&1,&2,&3],
                        vec![&0,&1,&3],
                        vec![&0,&2],
                        vec![&0,&2,&3],
                        vec![&0,&3],
                        vec![&1],
                        vec![&1,&2],
                        vec![&1,&2,&3],
                        vec![&1,&3],
                        vec![&2],
                        vec![&2,&3],
                        vec![&3]]);

Collect every subset of characters from the word "cat" into a HashSet of Strings.

static CHARS: &[char] = &['c', 'a', 't'];
let subsets: HashSet<String> = Subsets::of(CHARS).collect();
assert!(subsets.len() == 8);
// (Note that this is not the order in which the Subsets iterator
// generates items. It is merely convenient for checking the results.)
assert!(subsets.contains(""));
assert!(subsets.contains("c"));
assert!(subsets.contains("a"));
assert!(subsets.contains("t"));
assert!(subsets.contains("ca"));
assert!(subsets.contains("at"));
assert!(subsets.contains("ct"));
assert!(subsets.contains("cat"));

Methods

impl<'a, T: 'a, C: FromIterator<&'a T>> Subsets<'a, T, C>
[src]

Constructor.